;; --------------------------------
;; Presentación de entorno PlanTron
;; --------------------------------
;;  Este fichero recoge una serie de instrucciones
;;  que guían el aprendizaje del entorno de agentes
;;  PlanTron (ver fichero README).
;;
;;  Puedes ir copiando sobre una consola las
;;  instrucciones aquí descritas.
;;


;; Para cargar el entorno
;; ----------------------

;; Arrancamos en consola el CMUCL lisp con 'lisp' o 'cmucl'
#|

lisp

|#
;; También es posible usar SBCL lisp

;; Para salir del lisp usar la función 'quit':
#|

(quit)

|#


;; Cargamos el entorno en el lugar en que esté,
;; en el CC de la ETSII se encuentra en el
;; directorio /scratch/plantron
;;

(load "/scratch/plantron/pt-loader")

;; ¡Atención! Cargar siempre desde scratch, ya
;; que pueden haber modificaciones en los códigos.
;; Comprobar versión contenido fichero VERSION o
;; ejecutar (version-pt)



;; Partida con un humano
;; ---------------------

;; Movimientos de teclas (teclado numérico):
;;   8: arriba, 2:abajo, 6: derecha, 4:izquierda

;; ¡Atención! Asegurarse antes de jugar que el
;; tamaño de la terminal es al menos de 80x24
;; (VT100)
;; Trata de que no te maten

(pt:partida :rep 1)


;; Si durante la partida hay problemas de 
;; visualización como la desaparición de parte del
;; tablero (bordes, agentes, etc.) puede ser debido a
;; conflictos con el 'rlwrap', prueba a arrancar el
;; lisp mediante el comando 'lisp' o bien usa la
;; opción ":dev-vis" explicada en la sección "Pintar 
;; el tablero en otra terminal"

;; La expresión anterior es equivalente a:

(pt:partida "HUMANO" :rep 1)

;; Descripción de funciones y variables con describe

(describe 'pt:partida)


;; Información del agente "HUMANO"

(pt:info-jug "HUMANO")


;; Partida contra agentes "ESQUINA"
;; --------------------------------

(pt:partida "ESQUINA")


;; Más rápido, opción :sleep-cs <centesimas de segundo/turno>

(pt:partida "ESQUINA" :sleep-cs 1)


;; Utilización permanente de :sleep-cs

(setf pt-base:*sleep-cs* 15)


;; Consultar jugadores disponibles
;; -------------------------------

(pt:jug)

(describe 'pt:jug)

(pt:jug :full t)


;; Partida con agentes "BORDES", "IMITADOR" y "ESPIRAL"
;; -----------------------------------------

(pt:partida "BORDES")

(pt:partida "IMITADOR" :sleep-cs 5)

(pt:partida "ESPIRAL" :sleep-cs 5)

;; Dos tipos de agentes contra depredador

(pt:partida "BORDES" "IMITADOR" :rep 3 :sleep-cs 5)


;; Partida sin agente depredador
;; -----------------------------

(pt:partida "ESQUINA" :gen-dir nil :sleep-cs 2)

(pt:partida "BORDES" :gen-dir nil :sleep-cs 5)


;; Partida con agente "COBARDE" y "HORMIGA-BORRACHA"
;; -------------------------------------------------

(pt:partida "COBARDE" :sleep-cs 5)

(pt:partida "HORMIGA-BORRACHA" :sleep-cs 5)


;; Partidas cambiando el tipo de tablero
;; -------------------------------------

;; Tableros especificados con números

(pt:partida "BORDES" :tip-tab 1 :sleep-cs 5)

(pt:partida "ESQUINA" :tip-tab 11 :sleep-cs 15)

(pt:partida "ESQUINA" :tip-tab 13 :sleep-cs 5)


;; Tableros especificados con nombre de fichero

(pt:partida "HORMIGA-BORRACHA" :tip-tab "toronto2.txt" :sleep-cs 5)


;; Tableros aleatorios

(pt:partida "ESQUINA" :tip-tab t :sleep-cs 5)


;; Fijar permanentemente tipo de tablero al 18

(setf pt-base:*tip-tab* 18)


;; Fijar permanentemente tipo de tablero al por defecto

(setf pt-base:*tip-tab* nil)


;; Jugar con réplicas de agentes
;; -----------------------------

;; Jugar con 1 sola réplicasdel agente "ESQUINA"

(pt:partida "ESQUINA" :sleep-cs 5 :rep 1)


;; Jugar con 8 réplicas del agente "COBARDE"

(pt:partida "COBARDE" :sleep-cs 5 :rep 8)


;; Fijar permanentemente sólo 5 réplicas

(setf pt:*rep* 5)

(pt:partida "ESQUINA" :sleep-cs 1)


;; Partidas cambiando posiciones de inicio
;; ---------------------------------------

;; Posiciones de inicio aleatorias

(pt:partida "BORDES" :rep 5 :pos-alea t :sleep-cs 5)


;; Posiciones de inicio no aleatorias

(pt:partida "BORDES" :rep 5 :pos-alea nil :sleep-cs 5)


;; Fijar permanentemente posiciones aleatorias

(setf pt-base:*pos-alea* t)


;; Partidas con más energía inicial
;; --------------------------------

;; Fijar permanentemente la energía inicial a
;; la mitad de las casillas del tablero

(setf pt-base:*ratio-ene-jug-ini* 1.0)

(pt:partida "ESPIRAL" :gen-dir nil :sleep-cs 2)

(pt:partida "BORDES" :sleep-cs 5)


;; Partidas con más plantas iniciales
;; --------------------------------

(setf pt-agentes-genplc:*ratio-plantas* 0.2)

(describe 'pt-agentes-genplc:*ratio-plantas*)

(pt:partida "ESPIRAL" :sleep-cs 1)

(pt:partida "ESPIRAL" :sleep-cs 1  :gen-dir nil)


;; Partidas con agentes que no se cansan
;; -------------------------------------

;; El parámetro :prb-mov-cero a t hace que los
;; agentes no pierdan velocidad cuando están
;; bajos de energía

(pt:partida "ESPIRAL" :gen-dir nil :rep 1 :prb-mov-cero t :sleep-cs 2)

(pt:partida "ESPIRAL" :rep 1 :prb-mov-cero t :sleep-cs 2)

(pt:partida "BORDES" :rep 1 :prb-mov-cero t :sleep-cs 2)


;; Cargar y ejecutar un nuevo agente
;; ---------------------------------

(pt:load-ag "agentes-aluXXXX.lisp")

(describe 'pt:load-ag)

(pt:jug)

(pt:partida "ALUXXXX-ESQUINA" :sleep-cs 2)


;; Partida con trazas de consultas, opción :vis <nivel>
;; ----------------------------------------------------

(pt:partida "ESPIRAL" :sleep-cs 10 :vis 2)

(pt:partida "COBARDE" :sleep-cs 1 :vis 2)


;; Sin visualizador

(pt:partida "ESQUINA" :sleep-cs 1 :vis nil)


;; Utilización permanente de un visualizador

(setf pt:*vis* 2)

(pt:partida "COBARDE" :sleep-cs 1)


;; Pintar el tablero en otra terminal, opción :dev-vis
;; ---------------------------------------------------

;; Abrir una nueva terminal de al menos de 80x24 (VT100),
;; localizar con el comando 'tty' el dispositivo
;; asociado a nueva terminal (/dev/pts/?),
;; por ejemplo para "/dev/pts/2"

(pt:partida "ESPIRAL" :sleep-cs 1 :dev-vis "/dev/pts/2")


;; Traceamos función dir-posible-pref

(trace pt-agentes-simples::dir-posible-pref)

(pt:partida "ESPIRAL" :sleep-cs 1 :dev-vis "/dev/pts/2")

(untrace  pt-agentes-simples::dir-posible-pref)


;; Utilización permanente de otra terminal

(setf pt:*dev-vis* "/dev/pts/2")

(pt:partida "COBARDE" :sleep-cs 1)


;; Partidas paso a paso
;; --------------------

;; Activación de paso a paso (CTRL-C para abortar)

(setf pt-vis:*step* t)

(pt:partida "ESQUINA" :vis 2 :dev-vis "/dev/pts/2")

(pt:partida "IMITADOR" :vis 3 :dev-vis "/dev/pts/2")

(describe 'pt-vis:*step*)


;; Desactivación de paso a paso

(setf pt-vis:*step* nil)


;; Partidas de todos los agentes no interactivos
;; ---------------------------------------------

(pt:partida-todos)

(describe 'pt:partida-todos)


;; No esperar entre turnos (acelera las partidas)

(pt:partida-todos :sleep-nil t)


;; Resultados de grupos de 3 partidas con los agentes
;; "HORMIGA-BORRACHA" y "ESPIRAL" entre sí y ejecutadas
;; sin visualizador

(pt:partida-todos '("HORMIGA-BORRACHA" "ESPIRAL") :num 3 :pos-alea t :vis nil :sleep-nil t)

;; El resultado consta de sublistas para cada agente que
;; indican nombre, suma de número de turnos sobrevividos,
;; suma de la energia al final de las partidas y número de
;; partidas jugadas

;; Resultados de grupos de 20 partidas con todos los
;; agentes, posiciones y tableros aleatorios y ejecutadas
;; sin visualizador. Conviene realizarlas reiniciando el
;; lisp para que las partidas sean siempre las mismas

(pt:partida-todos :num 10 :pos-alea t :tip-tab t :vis nil :sleep-nil t)


;; Envío de agentes: trabajo con repositorio arena
;; -----------------------------------------------

;; Mostrar los agentes en repositorio arena

(pt-arena:ls)

(describe 'pt-arena:ls)

(pt-arena:ls "*" t)


;; Chequear y enviar agentes al repositorio arena
;; ¡Atención! Asegurarse de ejecutar esta instrucción
;; desde la máquina exthost (exthost.etsii.ull.es) y cargar
;; el entorno desde scratch (/scratch/plantron/pt-loader)

(pt-arena:send "agentes-aluXXXX.lisp")

;; Chequear cuidadosamente los mensajes y warnings emitidos
;; y asegurarse de que el fichero de agentes aparece listado
;; en el entorno arena. Si es necesario se puede reenviar
;; nuevamente el fichero borrándolo previamente mediante
;; pt-arena:del

(pt-arena:ls)


;; Cargar agente del repositorio arena

(pt-arena:load-ag "agentes-aluXXXX")


;; Eliminar un agente del repositorio arena
;; (no anula el envío)

(pt-arena:del "agentes-aluXXXX")

(describe 'pt-arena:del)


;; Tabla de resultados

(pt-arena:hof)

(describe 'pt-arena:hof)


;; Ejemplo de agentes buscadores de caminos
;; ----------------------------------------

; Cargar agentes de caminos del repositorio arena

(pt-arena:load-ag "agentes-aluXXXX-camino")


; Lanzar agente búsqueda de camino en anchura

(pt:partida "ALUXXXX-CAMINO-BA")

(pt:partida "ALUXXXX-CAMINO-BA" :vis 2 :rep 1)


; Lanzar agente búsqueda de camino en A*

(pt:partida "ALUXXXX-CAMINO-A")

(pt:partida "ALUXXXX-CAMINO-A" :vis 2 :rep 1)

(pt:partida "ALUXXXX-CAMINO-A" :sleep-nil t)

(setf pt-vis:*step* t)

(pt:partida "ALUXXXX-CAMINO-A" :vis 2 :rep 1 :dev-vis "/dev/pts/2")


;; Compilación de agentes
;; ----------------------

(compile-file "agentes-aluXXXX-camino.lisp")

(pt:load-ag "agentes-aluXXXX-camino.x86f")



;; Salir del lisp
;; --------------

(quit)
